DEF_ALIAS = {
   'window':  'WindowName',
   'open': 'VKOpenURL',
   'inputsource': 'ChangeInputSource',
}

KEYMAP_ALIAS = {
   'double':  'DoublePressModifier',
   'holding': 'HoldingKeyToKey',
   'press_modifier': 'KeyOverlaidModifier',
}

MODIFIER_ALIAS = {
    "shift"   : "SHIFT_L",
    "shift_l" : "SHIFT_L",
    "shift_r" : "SHIFT_R",
    "cmd"     : "COMMAND_L",
    "command" : "COMMAND_L",
    "cmd_l"   : "COMMAND_L",
    "cmd_r"   : "COMMAND_R",
    "opt"     : "OPTION_L",
    "option"  : "OPTION_L",
    "opt_l"   : "OPTION_L",
    "opt_r"   : "OPTION_R",
    "alt"     : "OPTION_L",
    "alt_l"   : "OPTION_L",
    "alt_r"   : "OPTION_R",
    "ctrl"    : "CONTROL_L",
    "control" : "CONTROL_L",
    "ctrl_l"  : "CONTROL_L",
    "ctrl_r"  : "CONTROL_R",
}

KEY_ALIAS = {
    "whitespace" : "SPACE",
    "sp"         : "SPACE",
    "del"        : "DELETE",
    "fdel"       : "FORWARD_DELETE",
    "esc"        : "ESCAPE",
    "caps"       : "CAPSLOCK",
    "left"       : "CURSOR_LEFT",
    "right"      : "CURSOR_RIGHT",
    "down"       : "CURSOR_DOWN",
    "up"         : "CURSOR_UP",
    "`"          : "BACKQUOTE",
    "1"          : "KEY_1",
    "2"          : "KEY_2",
    "3"          : "KEY_3",
    "4"          : "KEY_4",
    "5"          : "KEY_5",
    "6"          : "KEY_6",
    "7"          : "KEY_7",
    "8"          : "KEY_8",
    "9"          : "KEY_9",
    "0"          : "KEY_0",
    "-"          : "MINUS",
    "="          : "EQUAL",
    "["          : "BRACKET_LEFT",
    "]"          : "BRACKET_RIGHT",
    "\\"         : "BACKSLASH",
    ";"          : "SEMICOLON",
    "'"          : "QUOTE",
    ","          : "COMMA",
    "."          : "DOT",
    "/"          : "SLASH",

    "mouse_left"   : "PointingButton::LEFT",
    "mouse_right"  : "PointingButton::RIGHT",
    "mouse_middle" : "PointingButton::MIDDLE",
    "mouse1"       : "PointingButton::LEFT",
    "mouse2"       : "PointingButton::RIGHT",
    "mouse3"       : "PointingButton::MIDDLE",

    "scroll_up"    : "ScrollWheel::UP",
    "scroll_down"  : "ScrollWheel::DOWN",
    "scroll_left"  : "ScrollWheel::left",
    "scroll_right" : "ScrollWheel::right",
}

KEY_ALIAS.update(MODIFIER_ALIAS)